import * as angular from 'angular';
import * as _ from 'lodash';
import FilterWidgetModel from './filterWidget-model';

export default class FilterWidgetController {

    params: any;
    model: any;
    filterParams: any;
    isPopoverOpen = false;
    globalReset = false;

    static $inject = ['$scope', '$timeout'];

    constructor(private $scope, $timeout) {

        /**
         * Watch on the params coming into directive that populates filter widget.
         * Watch is used since params are coming from ASYNC call.
         */
        $scope.$watch(() => this.params, (newVal, oldVal) => {
            //we check if the model is already set.
            var hasModel = this.model;
            //get json format for newVal.
            var newValJson = newVal ? angular.toJson(newVal) : '';
            //get json format for oldVal
            var oldValJson = oldVal ? angular.toJson(oldVal) : '';
            //set isReset=true if we have different json objects coming.
            var isReset = (newValJson != oldValJson);
            //if globalReset=true, then we don't about comparison and just set isReset=true;
            if (this.globalReset) { isReset = true; }
            //we set a new model only when we have a newVal, AND, (newVal != oldVal)i.e. newParams are coming,
            // OR model is not defined.(params received first time)
            if (newVal && (isReset || !hasModel)) {
                this.setModel(newVal);
            }
        });

        //globalReset is used for cases when newVal, oldVal params are same, but still we want to reset the filter.
        //Special case, hence handled by an event.
        $scope.$on('resetFilterWidget', (event, value) => {
            this.globalReset = true;
        });

        // Connecting our outsideClick handler ourselves
        var bodyEl = document.body;
        var onBodyClick = (evt) => {
            /* I check the event's target has any parentage who's class begins with filter-
                that way I can differentiate any clicks that are not inside my filter widget, and cause
                the parent popover to come down.
                This selector here is - any element who's class attribute starts with filter-
             */

            var target = angular.element(evt.target);
            if (target.closest('[class^="filter-"]').length) {
                return;
            } else {
                this.isPopoverOpen = false;
                $timeout(() => $scope.$digest());
            }
        };

        bodyEl.addEventListener('click', onBodyClick);
        //Removing handler upon destruction
        $scope.$on('$destroy', function () {
            bodyEl.removeEventListener('click', onBodyClick);
        });
    }

    //sets the model to populate filter widget.
    setModel(newVal) {
        // creating a new model as soon as we have a set of params.
        this.model = new FilterWidgetModel(newVal);
        this.model.generateNewCriteria();

        // Creating the shallow copy of params that will be used to drive the logic of filter widget and the template.
        // We are creating a shallow copy of the passed-in params,so that we only change the params used to drive the logic when passed-in params have "really" changed.
        // Hitting "Refresh" button does not change the params, it just gives us another copy. Thus, filter state is not preserved on the template. and model does not match up.
        // Having a separate copy will ensure we have one copy of params used in filter widget until we do a  "Browser refresh", or passed-in params have "really" changed.
        this.filterParams = [];
        this.filterParams = angular.extend({}, this.filterParams, newVal);
    }

    /**
     * @description returns the current array of criterion
     * @returns {Array}
     */
    getCriteria() {
        if (this.model) {
            return this.model.getCriteria();
        } else {
            return [];
        }
    }

    /**
     * @description adds a new default criterion on click of "Add Filter" button
     */
    addCriterion() {
        this.model.addCriterion();
    }

    /**
     * @description deletes a criterion from current criterion, else deletes all the applied criterion. Invoked on clicking Trash icon.
     * @param id
     */
    deleteCriterion(id) {
        if (this.model.getCriteria().length > 1) {
            this.model.deleteCriterion(id);
        } else {
            this.deleteAppliedCriteria();
        }
    }

    /**
     * @description deletes all the criterion.
     */
    deleteAppliedCriteria() {
        //deleting the applied criteria
        this.model.deleteAppliedCriteria();
        //setting the filters to empty.
        this.apply();
        //Adds the first criteria as default.
        this.model.generateNewCriteria();
    }

    /**
     * @description applies the filter grid params on hitting Apply button.
     */
    apply() {

        this.model.apply();
        // fetching reference to grid params and updating them with new filters.
        var current = this.$scope.paramsCtrl.params;
        current.filter = this.createFilterParams();
        current.offset = 0;

        //closing the popover.
        this.isPopoverOpen = false;
    }

    /**
     * @description creates and then returns an array of comma separated criterion to be sent to filter grid params.
     * Iterates over applied Criteria since they contain all the applied criterion.
     * @returns {Array}
     */
    createFilterParams() {
        var filters = [];
        for (var idx = 0; idx < this.model.appliedCriteria.length; idx++) {
            var criterion = this.model.appliedCriteria[idx];
            filters.push(criterion.property.id + ',' + criterion.operator.value + ',' + criterion.value);
        }
        return filters;
    }

    /**
     * @description cancel on hitting 'Cancel' button.
     */
    cancel() {
        this.model.cancel();
        //closing the popover.
        this.isPopoverOpen = false;
    }

    /**
     * @description  method that return the options from filter config bundle based on the type passed, returns an object containing inputType and options.
     * @param paramType
     * @returns {Object}
     */
    getOperatorOptions(paramType) {
        return this.model.getOperatorOptions(paramType);
    }

    /**
     * @description generates the dropdown options for operators based on property selected.
     * @param id
     */
    updateCriterion(id) {
        this.model.updateCriterion(id);
    }

    /**
     * @description returns the number of the applied criteria (count for number of filters applied).
     * @returns {*}
     */
    getNumberOfAppliedFilters() {
        if (this.model && this.model.appliedCriteria) {
            return this.model.appliedCriteria.length;
        } else {
            return 0;
        }
    }

}