
import * as angular from 'angular';
import FilterConfigBundle from './filterWidget-config';


/**
 * @description This is the model for the filtering widget.
 * This code contains two major arrays known as newCriteria and appliedCriteria.
 * appliedCriteria: array of criterion(filter) that finally is submitted(sent) to grid params as filter params.
 * newCriteria: array of criteria that we work with i.e.temporary array of criteria.
 * On Initial setup of filtering widget, appliedCriteria and newCriteria both are empty.
 * We add a new default criteria in newCriteria. Subsequent criteria addition and deletion happen on newCriteria array only.
 * Only on Apply(submit), newCriteria is copied into appliedCriteria.
 */

export default class FilterWidgetModel {

    params: any;
    newCriteria: any;
    appliedCriteria: any;
    counter: number;

    constructor(params) {
        // params are the list of items to be filtered(passed through directive).
        this.params = params;
        // this array contains all the filter criterion that are being edited/added/deleted. (stores the temporary state of filter criterion).
        this.newCriteria = [];
        // this array contains all the filter criterion that are being applied/set. (stores the saved/final state of filter criterion).
        this.appliedCriteria = [];
        // counter is used set id value which will be used to delete a filter.
        this.counter = 1;
    }

    /**
     * @description this method adds a new default criterion
     */
    addCriterion() {
        // initially every new criterion being added, defaults to the first possible value in params passed. Based on first param, we set the operator value.
        var criterion = {
            id: this.counter++,
            property: this.params[0],
            operator: this.getOperatorOptions(this.params[0].type).options[0],
            value: ''
        };
        // adding the new criterion to the new criteria array.
        this.newCriteria.push(criterion);
    }

    /**
     * @description method to delete a criterion.
     * @param id
     */
    deleteCriterion(id) {
        var item = null;
        // Since during edit mode, everything is a part of new criteria, we search for the criterion in new criteria array by its id.
        this.newCriteria.some(function(iter) {                               //Find the item
            item = iter;
            return iter.id === id;
        });

        delete this.newCriteria[this.newCriteria.indexOf(item)];        //Delete the criterion
        this.newCriteria = filterUndefined(this.newCriteria);           //Clean up
    }


    /**
     * @description method to delete the applied criteria (used in reset).
     */
    deleteAppliedCriteria() {
        //setting to empty to delete the applied criteria.
        this.appliedCriteria = [];
        //setting new criteria to empty, to handle the case when reset is clicked during interaction(edit phase).
        this.newCriteria = [];
    }

    /**
     * @description method to set the operator and value of a particular criterion when property dropdown value is changed.
     * @param id
     */
    updateCriterion(id) {
        //finding the criterion in the existing list of criterion by id.
        var criterion = findItemById(this.newCriteria, id);
        // if the criterion exists, we reset the operator and value.
        if (criterion) {
            criterion.operator = this.getOperatorOptions(criterion.property.type).options[0];
            criterion.value = "";
        } else {
            console.log("ERROR : criterion not found ! dummy !");
        }
    }

    /**
     * @description method to generate new criterion.
     * This method is the start point for the entire filter widget logic.
     */
    generateNewCriteria() {
        var my = this;
        // check to see if an already applied criteria exists.
        if (this.appliedCriteria.length > 0) {
            // We create a copy of the applied criteria in the new criteria
            // Now, new criteria is the copy we play with, for all the interaction in the widget.
            // Applied criteria is touched/changed only when we Apply is clicked.
            this.newCriteria = $.extend(true, [], this.appliedCriteria);

            // iterating over new criteria and setting the property and operator from actual params being passed to have the same state.
            // As initially when we started filter widget, criteria and property were fetched from the actual params being passed in.
            this.newCriteria.forEach(function(criterion) {
                criterion.property = my.getProperty(criterion.property.id);
                criterion.operator = findItemByName(my.getOperatorOptions(criterion.property.type).options, criterion.operator.name);
            });
            //this.appliedCriteria = $.extend(true, [], this.appliedCriteria);

        } else {
            // There is no already applied criteria. Means filter is in reset state (first state).
            // set the new criterion to empty if it already exists.
            this.newCriteria = [];

            //Add the very first criterion to be displayed.
            this.addCriterion();
        }
    }

    /**
     * @description method to return the current criterion (edit mode).
     * @returns {Array}
     */
    getCriteria() {
        return this.newCriteria;
    }

    /**
     * @description method to cancel the applied criterion.
     */
    cancel() {
        //setting the new criterion to empty when cancel is clicked.
        // This means we don't want any interaction to persist.
        // So, we set the criterion to empty that contains the current state of all the criterions.
        this.newCriteria = [];
        // call generateCriteria that handles the state of the widget.
        this.generateNewCriteria();
    }

    /**
     * @description method to Apply(submit) the new criterion.
     */
    apply() {
        // We set whatever criterion exists in new criteria back to applied criterion to be saved.
        this.appliedCriteria = this.newCriteria;
    }

    /**
     * @description method that fetch the property from the original params passed into the widget.
     * @param id
     * @returns {*}
     */
    getProperty(id) {
        return findItemById(this.params, id);
    }

    /**
     * @description method that return the options from filter config bundle based on the type passed, returns an object containing inputType and options.
     * @param paramType
     * @returns {Object}
     */
    getOperatorOptions(paramType) {
        return FilterConfigBundle[paramType];
    }


}

/**
 * @description searches an item in array by its Id.
 * @param itemsArray
 * @param objid
 * @returns {*}
 */
function findItemById(itemsArray, objid) {
    var result = null;
    itemsArray.forEach(function(item) {
        if (item.id === objid) {
            result = item;
        }
    });

    return result;
}

/**
 * @description searches an item in the array by its name.
 * @param itemsArray
 * @param name
 * @returns {*}
 */
function findItemByName(itemsArray, name) {
    var result = null;
    itemsArray.forEach(function(item) {
        if (item.name === name) {
            result = item;
        }
    });

    return result;
}

/**
 * Filter and removes undefined items from the array so that
 * the view templates DON'T have to deal with it.
 */
function filterUndefined(array) {
    return array.filter(function(element) {
        return element !== undefined;
    });
}
