/**
 * DateUtils that provides global Date functions and constants
 *
 * @author: Ulhas Pai
 **/

var DateUtils = {

    HIGHCHARTS_DATE_FORMAT: '%I:%M %p, %b %d, %Y',       //Highcharts date format for sparkline charts.
    DATA_REFRESH_DATE_FORMAT: 'h:mm a MMM dd, yyyy',
    THIRTEEN_MONTH_HIGHCHARTS_DATE_FORMAT: '%b %d, %Y',
    THIRTEEN_MONTH_DATA_REFRESH_DATE_FORMAT: 'MMM dd, yyyy',

    /**
     * Given a date object, it will return an ISO 8601 String for the date
     * in the local time zone. Use this method if you don't want to use the
     * toISOString() method that returns the date in ISO 8601 String in UTC.
     *
     * @param date - the date object
     * @returns {string} - The ISO 8601 String in the local timezone
     */
    toISOStringLocal: function (date) {
        var timezone = -date.getTimezoneOffset(),
            dif = timezone >= 0 ? '+' : '-',
            pad = function (num) {
                var norm = Math.abs(Math.floor(num));
                return (norm < 10 ? '0' : '') + norm;
            };
        return date.getFullYear() +
            '-' + pad(date.getMonth() + 1) +
            '-' + pad(date.getDate()) +
            'T' + pad(date.getHours()) +
            ':' + pad(date.getMinutes()) +
            ':' + pad(date.getSeconds()) +
            dif + pad(timezone / 60) +
            ':' + pad(timezone % 60);
    },

    /**
     *
     * @param date
     */
    toHourFloor: function (date) {
        var d = new Date(date.getTime());
        d.setMinutes(0);
        d.setSeconds(0);
        d.setMilliseconds(0);
        return d;
    },

    /**
     *
     * @param date
     */
    toHourCeil: function (date) {
        var d = new Date(date.getTime());
        if (d.getMinutes() > 0 || d.getSeconds() > 0 || d.getMilliseconds() > 0) {
            d.setHours(d.getHours() + 1);
            d.setMinutes(0);
            d.setSeconds(0);
            d.setMilliseconds(0);
        }
        return d;
    },

    /**
     * Converts minutes to hours and minutes.
     * @param mins
     */
    getDisplayTextForMinutes: function (mins) {

        var units = {
            "year": 24 * 60 * 365,
            "month": 24 * 60 * 30,
            "week": 24 * 60 * 7,
            "day": 24 * 60,
            "hour": 60,
            "minute": 1
        };

        var result = [];

        if(mins === 0){
            result.push('< 1 min');
        } else{
            for (var name in units) {
                var p = Math.floor(mins / units[name]);
                if (p == 1) result.push(p + " " + name);
                if (p >= 2) result.push(p + " " + name + "s");
                mins %= units[name];
            }
        }
        return result.join(" ");
    },

    /**
     * Returns the extremes to be set for the x axis for the event time line from the specified
     * date time range
     *
     * @param dateTimeRange - the date time range for which the extremes are needed
     * @returns {*} - the extremes object that has the min and max property set for the extremes
     */
    getExtremesFromDateTimeRange: function (dateTimeRange) {
        var maxValueMillis = (new Date()).getTime();
        var minValue = new Date();
        var minValueMillis = 0;
        if (dateTimeRange.indexOf('LAST') >= 0) {
            var time = dateTimeRange.split('LAST_')[1];
            var factor = 1;
            if (time.indexOf('d') >= 0) {
                var days = parseInt(time.split('d')[0]);
                factor *= days * 24;
                minValueMillis = minValue.setHours(minValue.getHours() - factor);
            } else if (time.indexOf('h') >= 0) {
                var hours = parseInt(time.split('h')[0]);
                factor *= hours;
                minValueMillis = minValue.setHours(minValue.getHours() - factor);
            } else {
                console.log('Invalid dateTimeRange value ( ' + dateTimeRange + ' ). Returning extremes for LAST_72h.');
                return this.getExtremesFromDateTimeRange('LAST_72h');
            }
        } else {
            var dateStrings = dateTimeRange.split(',');
            minValueMillis = (new Date(dateStrings[0])).getTime();
            maxValueMillis = (new Date(dateStrings[1])).getTime();
        }
        return {min: minValueMillis, max: maxValueMillis};
    }

};

export = DateUtils;